package com.caimei.www.mapper;

import com.caimei.www.pojo.document.OssArchive;
import com.caimei.www.pojo.document.OssArchivePdf;
import com.caimei.www.pojo.document.OssAuthorization;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * Description
 *
 * @author : plf
 * @date : 2020/11/10
 */
@Mapper
public interface DocumentAuthDao {

    /**
     * 判断手机号是否有权限
     *
     * @param mobile
     * @return
     */
    OssAuthorization findOssAuthorizationByMobile(String mobile);

    /**
     * 查询资料列表
     *
     * @param name 商品名称，资料库名称，资料名称
     * @return
     */
    List<OssArchive> findOssArchive(String name);

    /**
     * 查询该资料库下的所有文件
     *
     * @param archiveId 资料库id
     * @return
     */
    List<OssArchivePdf> findOssArchivePdf(Integer archiveId);

    /**
     * 查询该文件资料
     *
     * @param pdfId
     * @return
     */
    OssArchivePdf findOssArchivePdfById(Integer pdfId);

    /**
     * 根据id获取资料库
     *
     * @param archiveId
     * @return
     */
    OssArchive getOssArchive(Integer archiveId);
}
